<?php
/**
 * @file
 * This file contains functions for openlucius groups.
 */

/**
 * Function to get all content within a group.
 *
 * @param \stdClass $group_node
 *   The group node.
 */
function openlucius_core_delete_group($group_node) {

  // Initialize operations array for batch.
  $operations = array();

  // Get the users in this active group.
  $users_in_group = openlucius_core_fetch_users('group', $group_node->nid);

  // Check the users.
  if (!empty($users_in_group)) {

    // Chunk the users in pieces.
    $user_chunks = array_chunk($users_in_group, 1, TRUE);

    // Loop through the chunks.
    foreach ($user_chunks as $chunk) {

      // Each chunk has x user objects.
      foreach ($chunk as $user_object) {

        // Add new operation to the batch.
        $operations[] = array(
          'openlucius_core_remove_users_from_group_in_batch',
          array(
            $group_node,
            $user_object,
          ),
        );
      }
    }
  }

  // Get all node ids that have the shared_group_reference $group_node->nid.
  $node_ids = db_select('field_data_field_shared_group_reference', 'g')
    ->fields('g', array('entity_id'))
    ->condition('g.field_shared_group_reference_nid', $group_node->nid, '=')
    ->execute()->fetchCol();

  // Check the node ids.
  if (count($node_ids) > 0) {

    // Break up all of our data so each process does not time out.
    $node_chunks = array_chunk($node_ids, 20);

    // Add new operation to the batch.
    foreach ($node_chunks as $key => $nid) {

      // Add new operation to the batch.
      $operations[] = array(
        'openlucius_core_remove_nodes_from_group_in_batch',
        array(
          $group_node,
          $nid,
        ),
      );

      // Add new operation to the batch.
      $operations[] = array(
        'openlucius_core_delete_group_notifications',
        array(
          $group_node,
          $nid,
        ),
      );

      // Add new operation to the batch.
      $operations[] = array(
        'openlucius_core_delete_group_notifications_notification',
        array(
          $group_node,
          $nid,
        ),
      );

      // Add new operation to the batch.
      $operations[] = array(
        'openlucius_core_delete_group_node_references',
        array(
          $group_node,
          $nid,
        ),
      );
    }
  }

  // Put all that information into our batch array.
  $batch = array(
    'operations'    => $operations,
    'title'         => t('Delete group'),
    'init_message'  => t('Initializing'),
    'error_message' => t('An error occurred'),
    'finished'      => 'openlucius_core_delete_group_finished',
  );

  // Start processing the batch operations.
  batch_set($batch);
}

/**
 * @param \stdClass $group_node
 *   The group node object from openlucius_core_node_delete().
 * @param \stdClass $user_object
 *   The user object
 * @param array $context
 *   The $context array gathers batch context information about the execution
 *   (read), as well as 'return values' for the current operation (write)
 */
function openlucius_core_remove_users_from_group_in_batch($group_node, $user_object, &$context) {
  // Set context message for the user to see.
  $context['message'] = t('Removing @user from group @group.', array(
    '@user'  => $user_object->name,
    '@group' => $group_node->title,
  ));
  openlucius_core_remove_users_from_group($group_node->nid, $user_object->uid, TRUE);
}

/**
 * @param \stdClass $group_node
 *   The group node object from openlucius_core_node_delete().
 * @param int $nid
 *   The node id of the node to delete.
 * @param array $context
 *   The $context array gathers batch context information about the execution
 *   (read), as well as 'return values' for the current operation (write)
 */
function openlucius_core_remove_nodes_from_group_in_batch($group_node, $nid, &$context) {
  // Set context message for the user to see.
  $context['message'] = t('Removing nodes from group @group.', array(
    '@group' => $group_node->title,
  ));
  node_delete_multiple($nid);
}

/**
 * Function for removing group notifications.
 *
 * @param \stdClass $group_node
 *   The group node object from openlucius_core_node_delete().
 * @param int $nid
 *   The node id to remove the notifications for.
 * @param array $context
 *   The $context array gathers batch context information about the execution
 *   (read), as well as 'return values' for the current operation (write)
 */
function openlucius_core_delete_group_notifications($group_node, $nid, &$context) {
  // Set context message for the user to see.
  $context['message'] = t('Removing notifications from group @group.', array(
    '@group' => $group_node->title,
  ));
  // Remove the notifications where the nid is $nid.
  db_delete('openlucius_notifications')
    ->condition('id', $nid, 'IN')
    ->condition('type', 'node', '=')
    ->execute();
}

/**
 * Function for removing group heartbeat activity.
 *
 * @param \stdClass $group_node
 *   The group node object from openlucius_core_node_delete().
 * @param int $nid
 *   The node id to remove the notifications for.
 * @param array $context
 *   The $context array gathers batch context information about the execution
 *   (read), as well as 'return values' for the current operation (write)
 */
function openlucius_core_delete_group_heartbeat_activity($group_node, $nid, &$context) {
  // Set context message for the user to see.
  $context['message'] = t('Removing activity from group @group.', array(
    '@group' => $group_node->title,
  ));
  // Remove the activity where the nid is $nid.
  db_delete('heartbeat_activity')
    ->condition('nid', $nid, 'IN')
    ->condition('type', 'node', '=')
    ->execute();
}

/**
 * Function for removing the notifications notification.
 *
 * @param \stdClass $group_node
 *   The group node object from openlucius_core_node_delete().
 * @param int $nid
 *   The node id to remove the notifications for.
 * @param array $context
 *   The $context array gathers batch context information about the execution
 *   (read), as well as 'return values' for the current operation (write)
 */
function openlucius_core_delete_group_notifications_notification($group_node, $nid, &$context) {
  // Set context message for the user to see.
  $context['message'] = t('Removing notifications from group @group.', array(
    '@group' => $group_node->title,
  ));
  // Remove the activity where the nid is $nid.
  db_delete('openlucius_notifications_notification')
    ->condition('nid', $nid, 'IN')
    ->execute();
}

/**
 * Function for removing the node reference.
 *
 * @param \stdClass $group_node
 *   The group node object from openlucius_core_node_delete().
 * @param int $nid
 *   The node id to remove the reference for.
 * @param array $context
 *   The $context array gathers batch context information about the execution
 *   (read), as well as 'return values' for the current operation (write)
 */
function openlucius_core_delete_group_node_references($group_node, $nid, &$context) {
  // Set context message for the user to see.
  $context['message'] = t('Removing node references from group @group.', array(
    '@group' => $group_node->title,
  ));
  // Remove the activity where the rid is $nid.
  db_delete('openlucius_core_node_references')
    ->condition('rid', $nid, 'IN')
    ->execute();
}

/**
 * This function is called by the batch 'finished' parameter.
 *
 * @param bool $success
 *   Whether or not the batch was successfull.
 * @param array $results
 *   The array of results gathered so far by the batch processing,
 *   for the current operation to append its own.
 * @param array $operations
 *   An array that can be used to store persistent data between iterations.
 */
function openlucius_core_delete_group_finished($success, $results, $operations) {
  $message = $success ? t('Bulkdelete perform§ed successfully.') : t('Bulkdelete has not been finished successfully.');
  watchdog('bulkdelete', $message);
}

/**
 * Function for group redirects based on enabled apps.
 *
 * @param array|bool $menu
 *   Either the menu item or FALSE.
 */
function openlucius_core_group_apps_redirect($menu) {

  // Group path restrictions based on enabled Apps.
  $default_types = openlucius_core_default_group_header_buttons(TRUE);
  $patterns = implode("\r\n", $default_types);
  $active_path = current_path();

  // Check if the active path matches a pattern.
  if (drupal_match_path($active_path, $patterns)) {

    // Get parts count.
    $parts = count($menu['map']) - 1;

    // Default menu map item.
    $nid = $menu['map'][$parts];

    // Check if this is an add for todos which is the only path not
    // using the last argument.
    if ($menu['page_callback'] == 'node_add' && $menu['page_arguments'][0] == 'ol_todo') {
      $nid = $menu['map'][$parts - 1];
    }

    // Check if we have a nid.
    if (!empty($nid)) {

      // Fetch group settings.
      $group_settings = variable_get('openlucius_core_group_settings', array());

      // Check if we have group settings.
      if (!empty($group_settings[$nid])) {

        // Fetch settings and buttons.
        $settings = $group_settings[$nid];
        $buttons  = openlucius_core_default_group_header_buttons();

        // Loop through and compare settings.
        foreach ($settings as $key => $setting) {

          // Check if setting is disabled.
          if ($setting === 0) {

            // Build patterns for matching.
            $patterns = implode("\r\n", $buttons[$key]);

            // Check if this path matches the patterns.
            if (drupal_match_path($active_path, $patterns)) {

              // Redirect to Dashboard.
              drupal_goto('node/' . $nid);
            }
          }
        }
      }
    }
  }
}

/**
 * Function for fetching the group_header_buttons.
 *
 * @param bool $paths_only
 *   Whether only the paths are required.
 *
 * @return array
 *   Returns an array with the button types which can be disabled.
 */
function openlucius_core_default_group_header_buttons($paths_only = FALSE) {
  // Buttons and the paths associated to these buttons.
  $group_buttons = array(
    'activity'       => array(
      'group-activity/*',
    ),
    'messages'       => array(
      'group-messages/*',
      'node/add/ol-message/*',
    ),
    'todos'          => array(
      'group-todo-lists/*',
      'node/add/ol-todo-list/*',
      'node/add/ol-todo/*',
    ),
    'text_documents' => array(
      'group-textdocuments/*',
      'node/add/ol-text-document/*',
    ),
    'files' => array(
      'node/add/ol-file-folder/*',
      'node/add/file/*',
    ),
  );

  // Allow other modules add add buttons to the default.
  drupal_alter('default_group_header_buttons', $group_buttons);

  // Check if the paths.
  if ($paths_only) {

    // Build paths array for matching.
    $final_paths = array();
    foreach ($group_buttons as $paths) {
      $final_paths = array_merge($final_paths, $paths);
    }
    return $final_paths;
  }

  return $group_buttons;
}

/**
 * Function for obtaining the default app blocks for the group dashboard.
 *
 * @return array
 *   Returns an array containing the key and blocks for a certain app.
 */
function openlucius_core_default_group_dashboard_blocks() {
  $app_blocks = array(
    'messages'       => 'messages_in_a_group-block_1',
    'todos'          => 'todos_in_group_dashboard-block',
    'files'          => 'ol_group_files',
    'text_documents' => 'text_documents_in_a_group-block_1',
  );

  // Allow other modules to add new app blocks.
  drupal_alter('openlucius_core_default_group_dashboard_blocks', $app_blocks);

  return $app_blocks;
}
